import boto3
import sys
import os
from tqdm import tqdm
from rich.console import Console
console = Console()

from dguard_oss.config import load_config

def pprint(message):
    if "ERR" in message:
        console.print(message, style="bold red")
    elif "URL" in message or "FILE" in message:
        console.print(message, style="bold yellow underline")
    elif "Usage" in message:
        print(message)
    else:
        console.print(message, style="bold green")


# Progress bar callbacks
class Progress:
    def __init__(self, total_size):
        self._progress = tqdm(total=total_size, unit="B", unit_scale=True, unit_divisor=1024)

    def update(self, bytes_amount):
        self._progress.update(bytes_amount)

    def close(self):
        self._progress.close()


def upload(file_path, bucket_name, object_name, config_id):
    # 这里暂时没有使用 config_id，可根据实际需求修改
    s3_client = boto3.client(
        's3',
        endpoint_url=config_id['endpoint'],
        aws_access_key_id=config_id['access_key'],
        aws_secret_access_key=config_id['secret_key']
    )
    try:
        # 检查存储桶是否存在，如果不存在，尝试创建
        try:
            s3_client.head_bucket(Bucket=bucket_name)
        except Exception:
            confirm = input(f"Bucket '{bucket_name}' does not exist. Create it? (yes/no): ").strip().lower()
            if confirm == "yes":
                s3_client.create_bucket(Bucket=bucket_name)
            else:
                pprint("Operation aborted.")
                return

        file_size = os.path.getsize(file_path)
        progress = Progress(file_size)

        def upload_callback(bytes_amount):
            progress.update(bytes_amount)

        with open(file_path, 'rb') as file_data:
            s3_client.upload_fileobj(
                file_data,
                bucket_name,
                object_name,
                Callback=upload_callback
            )
        progress.close()

        pprint(f"[OSS] File '{file_path}' uploaded to bucket '{bucket_name}' as '{object_name}'.")
        pprint(f"[URL] {config_id['endpoint']}/{bucket_name}/{object_name}")
    except Exception as e:
        pprint(f"[OSS ERROR!] Upload failed: {e}")


def download(bucket_name, object_name, save_path, config_id):
    s3_client = boto3.client(
        's3',
        endpoint_url=config_id['endpoint'],
        aws_access_key_id=config_id['access_key'],
        aws_secret_access_key=config_id['secret_key']
    )

    try:
        # 获取对象信息以确定文件大小
        object_info = s3_client.head_object(Bucket=bucket_name, Key=object_name)
        file_size = object_info['ContentLength']
        progress = Progress(file_size)

        def download_callback(bytes_amount):
            progress.update(bytes_amount)

        with open(save_path, 'wb') as file_data:
            s3_client.download_fileobj(
                bucket_name,
                object_name,
                file_data,
                Callback=download_callback
            )
        progress.close()

        pprint(f"[OSS] File '{object_name}' from bucket '{bucket_name}' downloaded to '{save_path}'.")
    except Exception as e:
        pprint(f"[OSS ERROR!] Download failed: {e}")


def show_help():
    help_text = """
Usage:
  oss upload <file_path> [bucket] [object_name] [--id=<config_id>]
  oss download [bucket] <object_name> <save_path> [--id=<config_id>]
  oss help
  oss-config
  oss-config-new

Options:
  [--id=<config_id>]  Specify configuration ID. Defaults to 'config'.
  * oss-config: Display the configuration for a given ID.
  * oss-config-new: Create a new configuration file.
"""
    print(help_text)


def main():
    if len(sys.argv) < 2:
        show_help()
        sys.exit(1)

    command = sys.argv[1]
    config_id = load_config()

    # Extract --id option
    for arg in sys.argv:
        if arg.startswith("--id="):
            parts = arg.split("=", 1)
            if len(parts) == 2:
                config_id = load_config(parts[1])

    if command == 'upload':
        if len(sys.argv) < 4:
            pprint("Error: Invalid arguments for 'upload'.")
            show_help()
            sys.exit(1)

        file_path = sys.argv[2]
        bucket_name = sys.argv[3] if len(sys.argv) > 3 else "temp"
        object_name = sys.argv[4] if len(sys.argv) > 4 else os.path.basename(file_path)
        upload(file_path, bucket_name, object_name, config_id)

    elif command == 'download':
        if len(sys.argv) < 4:
            pprint("Error: Invalid arguments for 'download'.")
            show_help()
            sys.exit(1)

        bucket_name = sys.argv[2]
        object_name = sys.argv[3]
        save_path = sys.argv[4]
        download(bucket_name, object_name, save_path, config_id)

    elif command == 'help':
        show_help()

    else:
        pprint(f"Unknown command: {command}")
        show_help()
        sys.exit(1)


if __name__ == "__main__":
    main()
    # config = load_config("config")
    # upload("/Users/zhaosheng/zhaosheng.sh", "temp", "zhaosheng.a",config)


from setuptools import setup, find_packages

with open("requirements.txt", encoding="utf8") as f:
    requirements = f.readlines()

setup(
    name='dguard_oss',
    version=open("VERSION", encoding="utf8").read(),
    description='A command-line tool for interacting with MinIO.',
    author="Zhao Sheng",
    author_email="zhaosheng@lyxxkj.com.cn",
    long_description=open("README.md", encoding="utf8").read(),
    long_description_content_type="text/markdown",

    url='https://github.com/nuaazs/dguard_oss',
    packages=find_packages(),
    install_requires=requirements,
    entry_points={
        'console_scripts': [
            'oss=dguard_oss.cli:main',
            'oss-config=dguard_oss.config:check_config_cli',
            'oss-config-new=dguard_oss.config:create_new_config_cli',
        ]
    },
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.6',
    include_package_data=True,

)



import os
import json

from rich.console import Console
console = Console()

def print(message):
    if "ERR" in message:
        console.print(message, style="bold red")
    else:
        console.print(message, style="bold green")


CONFIG_DIR = os.path.expanduser("~/.dguard_oss")
DEFAULT_ID = "config"

# Ensure the configuration directory exists
def ensure_config_dir():
    if not os.path.exists(CONFIG_DIR):
        os.makedirs(CONFIG_DIR)

# Load a configuration file based on the ID
def load_config(config_id=DEFAULT_ID):
    ensure_config_dir()
    config_path = os.path.join(CONFIG_DIR, f"{config_id}.json")

    if not os.path.exists(config_path):
        # try to find in /opt/.dguard_oss
        config_path_new = os.path.join("/opt/.dguard_oss", f"{config_id}.json")
        if os.path.exists(config_path_new):
            config_path = config_path_new
        else:
            print(f"[OSS ERROR!] Configuration file '{config_path}' does not exist.")
            create_new_config(config_path)

    with open(config_path, 'r') as f:
        config = json.load(f)
        if "http" not in config["endpoint"]:
            config["endpoint"] = "http://" + config["endpoint"]
        # print(f"Loaded configuration for ID '{config_id}': {config}")
        return config

# Create a new configuration file

def create_new_config(config_path):
    print("Creating a new configuration file.")
    endpoint = input("Enter MinIO endpoint (e.g., play.min.io:9000): ")
    access_key = input("Enter MinIO access key: ")
    secret_key = input("Enter MinIO secret key: ")
    secure = input("Use HTTPS? (yes/no): ").strip().lower() == "yes"

    config = {
        "endpoint": endpoint,
        "access_key": access_key,
        "secret_key": secret_key,
        "secure": secure
    }

    with open(config_path, 'w') as f:
        json.dump(config, f, indent=4)
        print(f"Configuration saved to '{config_path}'.")


def create_new_config_cli():
    ensure_config_dir()
    config_id = input("Enter a configuration ID: ")
    config_path = os.path.join(CONFIG_DIR, f"{config_id}.json")
    create_new_config(config_path)

def check_config_cli():
    ensure_config_dir()
    config_id = input("Enter a configuration ID: ")
    config_path = os.path.join(CONFIG_DIR, f"{config_id}.json")
    a = load_config(config_id)
    print(a)

    